<?php

namespace App\Domain\Orders\Tests\Factories\Orders;

use Ensi\LaravelTestFactories\BaseApiFactory;
use Ensi\OmsClient\Dto\OrderItem;
use Ensi\OmsClient\Dto\OrderItemChangeQtyResponse;

class OrderItemFactory extends BaseApiFactory
{
    protected function definition(): array
    {
        $pricePerOne = $this->faker->numberBetween(1, 10000);
        $costPerOne = $this->faker->numberBetween($pricePerOne, $pricePerOne + 1000);

        $qty = $this->faker->randomFloat(2, 1, 100);

        return [
            'id' => $this->faker->modelId(),
            'order_id' => $this->faker->modelId(),
            'offer_id' => $this->faker->modelId(),
            'shipment_id' => $this->faker->modelId(),
            'name' => $this->faker->text(50),
            'qty' => $qty,
            'price' => round($pricePerOne * $qty),
            'price_per_one' => $pricePerOne,
            'cost' => round($costPerOne * $qty),
            'cost_per_one' => $costPerOne,
            'refund_qty' => $this->faker->randomFloat(2, 0, $qty),
            'product_weight' => $this->faker->numberBetween(1, 100),
            'product_weight_gross' => $this->faker->numberBetween(1, 100),
            'product_width' => $this->faker->numberBetween(1, 100),
            'product_height' => $this->faker->numberBetween(1, 100),
            'product_length' => $this->faker->numberBetween(1, 100),
            'product_barcode' => $this->faker->nullable()->numerify('############'),
            'is_added' => $this->faker->boolean(),
            'is_deleted' => $this->faker->boolean(),
            'created_at' => $this->faker->dateTime(),
            'updated_at' => $this->faker->dateTime(),
        ];
    }

    public function make(array $extra = []): OrderItem
    {
        return new OrderItem($this->makeArray($extra));
    }

    public function makeResponseChange(array $extra = []): OrderItemChangeQtyResponse
    {
        return new OrderItemChangeQtyResponse(['data' => [$this->make($extra)]]);
    }
}
