<?php

use App\Domain\Catalog\Tests\Factories\Feeds\ActualCategoryPropertyFactory;
use App\Http\ApiV1\Support\Tests\ApiV1ComponentWithRightsTestCase;
use Ensi\AdminAuthClient\Dto\RightsAccessEnum;

use function Pest\Laravel\getJson;
use function Pest\Laravel\postJson;

uses(ApiV1ComponentWithRightsTestCase::class);
uses()->group('component', 'catalog', 'properties');

test('POST /api/v1/catalog/feeds/actual-category-properties:search success', function (int $right) {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights([$right]);

    $this->mockFeedActualCategoryPropertiesApi()->allows([
        'searchActualCategoryProperties' => ActualCategoryPropertyFactory::new()->makeResponseSearch([
            ['id' => 33],
            ['id' => 34],
        ], 2),
    ]);

    $request = [
        'filter' => ['actual_category_property_id' => 1],
    ];

    postJson('/api/v1/catalog/feeds/actual-category-properties:search', $request)
        ->assertOk()
        ->assertJsonCount(2, 'data')
        ->assertJsonStructure(['data' => [['id', 'actual_category_property_id']]])
        ->assertJsonPath('data.0.id', 33);
})->with([
    RightsAccessEnum::TECHNICAL_TABLES_READ,
    RightsAccessEnum::TECHNICAL_TABLES_ALL,
]);

test('POST /api/v1/catalog/feeds/actual-category-properties:search forbidden error', function () {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights([]);

    $request = [
        'filter' => ['actual_category_property_id' => 1],
    ];

    postJson('/api/v1/catalog/feeds/actual-category-properties:search', $request)
        ->assertForbidden();
});

test('GET /api/v1/catalog/feeds/actual-category-properties:meta success', function (int $right) {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights([$right]);

    getJson('/api/v1/catalog/feeds/actual-category-properties:meta')
        ->assertOk()
        ->assertJsonStructure(['data' => ['fields', 'detail_link', 'default_sort', 'default_list']]);
})->with([
    RightsAccessEnum::TECHNICAL_TABLES_READ,
    RightsAccessEnum::TECHNICAL_TABLES_ALL,
]);

test('GET /api/v1/catalog/feeds/actual-category-properties:meta forbidden error', function () {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights([]);

    getJson('/api/v1/catalog/feeds/actual-category-properties:meta')
        ->assertForbidden();
});
