<?php

namespace App\Http\ApiV1\Modules\Customers\Controllers;

use App\Domain\Common\Data\Meta\Enum\Customers\CustomerUserEnumInfo;
use App\Domain\Common\Data\Meta\Field;
use App\Http\ApiV1\Modules\Customers\Queries\CustomersConfirmationCodesQuery;
use App\Http\ApiV1\Modules\Customers\Resources\CustomerConfirmationCodesResource;
use App\Http\ApiV1\Support\Resources\ModelMetaResource;
use Illuminate\Contracts\Support\Responsable;

class CustomerConfirmationCodesController
{
    public function search(CustomersConfirmationCodesQuery $query): Responsable
    {
        return CustomerConfirmationCodesResource::collectPage($query->get());
    }

    public function meta(
        CustomerUserEnumInfo $customerUserEnumInfo,
    ): ModelMetaResource {
        return new ModelMetaResource([
            Field::id()->listDefault()->filterDefault(),
            Field::enum('user_id', __('meta.customers.user_id'), $customerUserEnumInfo)->listDefault()->filterDefault(),
            Field::phone('phone', __('meta.customers.phone'))->listDefault()->filterDefault(),
            Field::integer('attempt', __('meta.confirmation_codes.attempt'))->listDefault(),
            Field::integer('failed_checks_count', __('meta.confirmation_codes.failed_checks_count'))->listDefault(),

            Field::datetime('expired_at', __('meta.confirmation_codes.expired_at'))->listDefault(),
            Field::datetime('verified_at', __('meta.confirmation_codes.verified_at'))->listDefault(),
            Field::datetime('created_at', __('meta.created_at'))->listDefault(),
            Field::datetime('updated_at', __('meta.updated_at'))->listDefault(),
        ]);
    }
}
