<?php

use App\Domain\Customers\Tests\Factories\CustomerConfirmationCodeFactory;
use App\Http\ApiV1\Support\Tests\ApiV1ComponentWithRightsTestCase;
use Ensi\AdminAuthClient\Dto\RightsAccessEnum;

use function Pest\Laravel\getJson;
use function Pest\Laravel\postJson;

uses(ApiV1ComponentWithRightsTestCase::class);
uses()->group('component');

test('POST /api/v1/customers/confirmation-codes:search 200', function () {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights([RightsAccessEnum::TECHNICAL_TABLES_READ]);

    $this->mockCustomersAuthConfirmationCodesApi()->allows([
        'searchConfirmationCodes' => CustomerConfirmationCodeFactory::new()->makeResponseSearch([
            ['id' => 2],
            ['id' => 3],
        ], 2),
    ]);

    $request = [
        'filter' => ['name' => 'foo'],
    ];

    postJson('/api/v1/customers/confirmation-codes:search', $request)
        ->assertOk()
        ->assertJsonCount(2, 'data')
        ->assertJsonStructure(['data' => [[ 'id', 'user_id', 'phone', 'attempt', 'failed_checks_count', 'expired_at', 'verified_at', 'created_at', ]]])
        ->assertJsonPath('data.0.id', 2);
});

test('POST /api/v1/customers/confirmation-codes:search 403', function () {
    postJson("/api/v1/customers/confirmation-codes:search")
        ->assertForbidden();
});

test('GET /api/v1/customers/confirmation-codes:meta 200', function () {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights([RightsAccessEnum::TECHNICAL_TABLES_READ]);

    getJson('/api/v1/customers/confirmation-codes:meta')
        ->assertOk();
});

test('GET /api/v1/customers/confirmation-codes:meta 403', function () {
    getJson("/api/v1/customers/confirmation-codes:meta")
        ->assertForbidden();
});
