<?php

use App\Domain\Customers\Tests\Factories\DeletingCustomersFactory;
use App\Http\ApiV1\Support\Tests\ApiV1ComponentWithRightsTestCase;
use Ensi\AdminAuthClient\Dto\RightsAccessEnum;

use function Pest\Laravel\getJson;
use function Pest\Laravel\postJson;

uses(ApiV1ComponentWithRightsTestCase::class);
uses()->group('component');

test('POST /api/v1/customers/deleting-customers:search 200', function () {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights([RightsAccessEnum::TECHNICAL_TABLES_READ]);

    $this->mockCustomersDeletingCustomersApi()->allows([
        'searchDeletingCustomers' => DeletingCustomersFactory::new()->makeResponseSearch([
            ['id' => 2],
            ['id' => 3],
        ], 2),
    ]);

    postJson('/api/v1/customers/deleting-customers:search')
        ->assertStatus(200)
        ->assertJsonCount(2, 'data')
        ->assertJsonPath('data.0.id', 2);
});

test('POST /api/v1/customers/deleting-customers:search 403', function () {
    postJson("/api/v1/customers/deleting-customers:search")
        ->assertForbidden();
});

test('GET /api/v1/customers/deleting-customers:meta 200', function () {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights([RightsAccessEnum::TECHNICAL_TABLES_READ]);

    getJson('/api/v1/customers/deleting-customers:meta')
        ->assertStatus(200);
});

test('POST /api/v1/customers/deleting-customers:meta 403', function () {
    getJson("/api/v1/customers/deleting-customers:meta")
        ->assertForbidden();
});
