<?php

use App\Http\ApiV1\Modules\Logistic\Tests\DeliveryServices\Factories\AddPaymentMethodsToDeliveryServiceRequestFactory;
use App\Http\ApiV1\Modules\Logistic\Tests\DeliveryServices\Factories\DeletePaymentMethodFromDeliveryServiceRequestFactory;
use App\Http\ApiV1\Support\Tests\ApiV1ComponentWithRightsTestCase;
use Ensi\AdminAuthClient\Dto\RightsAccessEnum;

use function Pest\Laravel\postJson;

uses(ApiV1ComponentWithRightsTestCase::class);
uses()->group('component', 'logistic');

test("POST /api/v1/logistic/delivery-services/{id}:add-payment-methods success", function (array $accessRights) {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights($accessRights);

    $deliveryServiceId = 1;
    $addPaymentMethodsToDeliveryServiceData = AddPaymentMethodsToDeliveryServiceRequestFactory::new()->make();

    $this->mockLogisticDeliveryServicesApi()->shouldReceive('addPaymentMethodsToDeliveryService');

    postJson("/api/v1/logistic/delivery-services/{$deliveryServiceId}:add-payment-methods", $addPaymentMethodsToDeliveryServiceData)
        ->assertStatus(200)
        ->assertJsonPath('data', null);
})->with([
    [[RightsAccessEnum::DELIVERY_SERVICES_PAYMENT_METHOD_EDIT]],
]);

test("POST /api/v1/logistic/delivery-services/{id}:add-payment-methods 403", function () {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRoles([]);

    $deliveryServiceId = 1;
    $addPaymentMethodsToDeliveryServiceData = AddPaymentMethodsToDeliveryServiceRequestFactory::new()->make();

    postJson("/api/v1/logistic/delivery-services/{$deliveryServiceId}:add-payment-methods", $addPaymentMethodsToDeliveryServiceData)
        ->assertStatus(403);
});

test("POST /api/v1/logistic/delivery-services/{id}:delete-payment-method success", function (array $accessRights) {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRights($accessRights);

    $deliveryServiceId = 1;
    $deletePaymentMethodFromDeliveryServiceData = DeletePaymentMethodFromDeliveryServiceRequestFactory::new()->make();

    $this->mockLogisticDeliveryServicesApi()->shouldReceive('deletePaymentMethodFromDeliveryService');

    postJson("/api/v1/logistic/delivery-services/{$deliveryServiceId}:delete-payment-method", $deletePaymentMethodFromDeliveryServiceData)
        ->assertStatus(200)
        ->assertJsonPath('data', null);
})->with([
    [[RightsAccessEnum::DELIVERY_SERVICES_PAYMENT_METHOD_EDIT]],
]);

test("POST /api/v1/logistic/delivery-services/{id}:delete-payment-method 403", function () {
    /** @var ApiV1ComponentWithRightsTestCase $this */
    $this->setRoles([]);

    $deliveryServiceId = 1;
    $deletePaymentMethodFromDeliveryServiceData = DeletePaymentMethodFromDeliveryServiceRequestFactory::new()->make();

    postJson("/api/v1/logistic/delivery-services/{$deliveryServiceId}:delete-payment-method", $deletePaymentMethodFromDeliveryServiceData)
        ->assertStatus(403);
});
