<?php

namespace App\Http\ApiV1\Support\Resources;

use App\Http\ApiV1\Support\Pagination\Page;
use DateTime;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Http\Resources\MissingValue;

abstract class BaseJsonResource extends JsonResource
{
    public const DATE_TIME_FORMAT = 'Y-m-d\TH:i:s.u\Z';
    public const DATE_FORMAT = 'Y-m-d';

    public function dateTimeToIso(?DateTime $datetime): ?string
    {
        return $datetime?->format(static::DATE_TIME_FORMAT);
    }

    public function dateToIso(?DateTime $date): ?string
    {
        return $date?->format(static::DATE_FORMAT);
    }

    public static function collectionWithPagination($resource, array $pagination): AnonymousResourceCollection
    {
        $collection = static::collection($resource);
        $currentAdditional = $collection->additional ?: [];
        $append = ['meta' => ['pagination' => $pagination ?: null]];

        return static::collection($resource)->additional(array_merge_recursive($currentAdditional, $append));
    }

    public static function collectPage(Page $page): AnonymousResourceCollection
    {
        return static::collectionWithPagination($page->items, $page->pagination);
    }

    protected static function makeIfNotNull($value): ?JsonResource
    {
        return $value ? parent::make($value) : null;
    }

    protected function whenNotNull($value, $default = null)
    {
        if (!is_null($value)) {
            return value($value);
        }

        return func_num_args() === 2 ? value($default) : new MissingValue();
    }

    protected function fileUrl($file): ?string
    {
        if (is_null($file)) {
            return null;
        }

        return $file->getUrl();
    }
}
