<?php

namespace App\Http\ApiV1\Support\Resources;

use App\Domain\Common\Data\Meta\Fields\AbstractField;
use Illuminate\Contracts\Support\Responsable;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;

class ModelMetaResource implements Responsable
{
    /**
     * @param AbstractField[] $fields
     */
    public function __construct(protected array $fields)
    {
    }

    /**
     * @param  Request  $request
     */
    public function toResponse($request): Response
    {
        $defaultSort = null;
        $detailLink = null;
        $defaultList = [];
        $defaultFilter = [];
        foreach ($this->fields as $field) {
            if ($field->isSortDefault()) {
                $defaultSort = $field->sortDefaultDirection === 'desc' ? '-' . $field->code : $field->code;
            }
            if ($field->isDetailLink()) {
                $detailLink = $field->code;
            }
            if ($field->isListDefault()) {
                $defaultList[] = $field->code;
            }
            if ($field->isFilterDefault()) {
                $defaultFilter[] = $field->code;
            }
        }

        return response()->json(['data' => [
            'fields' => $this->fields,
            'detail_link' => $detailLink,
            'default_sort' => $defaultSort,
            'default_list' => $defaultList,
            'default_filter' => $defaultFilter,
        ]]);
    }
}
